function simSpheresStudentVer
% Simulate three sphere in a "bowl" formed by three planar half-spaces.
% The number of sphere is fixed at three and the number of half-spaces is
% also fixed at three.  There are 12 pairs of possible contacts.  The gap
% functions can be written in closed form and can be applied at every time
% step, so this is done to simplify the code.

%-----------------------------------------
% Set up params that define the simulation
sim.t0 = 0;     % Initial simluation time
sim.time = 0;   % Current simluation time
sim.h = 0.01;  % Simulation time step
sim.T = 1;    % Final simulation time  
sim.nd = 3;     % Number of friction directions.  CAN BE VARIED!!

%-----------------------------------------
% Set up body data structure.  This also sets the initial conditions.
nBodies = 3;    % Fixed number of bodies
nConts = 12;    % Fixed number of contacts (i.e., number of body pairs)
[body] = initBodies();

% Convenient to hyave copies of these vactors for state updates in
% simulation loop.
nu_ell = [body(1).nu; body(2).nu; body(3).nu];
u_ell =  [body(1).u;  body(2).u;  body(3).u];

%-----------------------------------------
% Create arrays of solution for ploting trajectories
%    1:1 ---------------- kinetic energy
%    2:2 ---------------- potential energy
%    3:20 --------------- pext
%   21:38 --------------- Gn * pn
%   39:56 --------------- Gf * pf
%   57:74 --------------- nu
%   75:95 --------------- u
%   96:107 -------------- Psi_n
%  108:119 -------------- s
%  120:131 -------------- pn
%  132:131 + 12*sim.nd -- pf
nRows =  nConts * (3 + sim.nd) + 4 * (6 * nBodies) + 7 * nBodies + 2;
time = sim.t0 : sim.h : sim.T;
nCols = length(time);
trajAll = zeros(nRows, nCols);

%-----------------------------------------
% Store the initial conditionni the array for plotting.
% No impulses have been computed yet, so just store everything else.
M = makeM(body); % Because bodies are spheres, M is constant for all time.
Minv = inv(M);   % Also constant

% Kinetic energy
KE = nu_ell' * M * nu_ell / 2;
trajAll(1,1) = KE;

% Potential energy
grav = makeGrav;  % Get the gravity vector
PE = -grav' * (body(1).u(1:3) * body(1).mass ...
             + body(2).u(1:3) * body(2).mass ...
             + body(3).u(1:3) * body(3).mass);
trajAll(2,1) = PE;

% External Impulse
trajAll(3:20, 1) = makePext(body, sim);

% velocities and configurations of bodies
trajAll(57:74, 1) = nu_ell;
trajAll(75:95, 1) = u_ell;

% Distances at contacts
[Psi] = makePsi(body, sim);
trajAll(96:107, 1) = Psi.N;

%-----------------------------------------
% Start the simulation
ell = 0;
% Because all contacts are tracked every time step, U and E are constant
U = makeU(body);  % U is constant for entire simulation
E = makeE(sim);   % E is constant for entire simulation
for i = 1 : nCols 
    [Gn, Gf] = makeG(body, sim); 
    [Psi] = makePsi(body, sim);
    pext = makePext(body, sim);       % Input to drive the dynamics
    V = makeV(body);                  % Needed for configuration update

    MinvGn = Minv*Gn;
    MinvGf = Minv*Gf;
    MinvPext = Minv*pext;
    
    MM = [];  % Fill in this formula
    
    qq = [];  % Fill in this formula
            
    lcpSoln0 = zeros(length(qq), 1);       % Initial guess of solution
    [lcpSoln,err] = lemke(MM,qq,lcpSoln0); % Solve the subproblem
    
    pn_ellp1 = lcpSoln(1 : 12);
    pf_ellp1 = []; % Fill in this formula
    s_ellp1  = []; % Fill in this formula
    
    % Update velocities and configurations of the bodies and the time.
    nu_ellp1 = []; % Fill in this formula
    u_ellp1 = [];  % Fill in this formula

    % Update the body data structure
    for j = 1 : 3
        % Renormalize unit quaternions first
        quat = []; % Fill in this formula
        uQuat = []; % Fill in this formula
        q_ellp1(j*7-3 : j*7) = uQuat;
        body(j).u  = []; % Fill in this formula
        body(j).nu = []; % Fill in this formula
    end
    
    % Update the trajectory information
    % Kinetic energy
    KE = []; % Fill in this formula

    % Potential energy
    grav = makeGrav;
    PE = []; % Fill in this formula
    
    % Net contact impulses
    netNormImp = []; % Fill in this formula
    netFricImp = []; % Fill in this formula
    
    % Distances at contacts
    [Psi] = makePsi(body, sim);  % Psi.N is the vector of distances

    % Update trajectory array with the new values.
    %    1:1 ---------------- kinetic energy
    %    2:2 ---------------- potential energy
    %    3:20 --------------- pext
    %   21:38 --------------- Gn * pn
    %   39:56 --------------- Gf * pf
    %   57:74 --------------- nu
    %   75:95 --------------- u
    %   96:107 -------------- Psi_n
    %  108:119 -------------- s
    %  120:131 -------------- pn
    %  132:131 + 12*sim.nd -- pf
    trajAll(:,i) = [KE;  PE;  pext;  netNormImp;  netFricImp; nu_ellp1; 
                    u_ellp1;  Psi.N;  s_ellp1;  pn_ellp1;  pf_ellp1];

    % Prepare for next iteration of the simulation
    nu_ell = nu_ellp1;
    u_ell  = u_ellp1;
    ell = ell + 1;
    sim.time = sim.time + sim.h;
end
plotAll(time, trajAll, sim);
end

function [body] = initBodies()
% Assume three moving spheres
body(1).radius = 1;
body(1).mass = 4 / 3 * pi * body(1).radius^3;
body(1).type = 'sphere';
body(1).mom = 2 / 5 * body(1).mass * body(1).radius^2 * eye(3, 3);
body(1).u = [5; 0; 4; 1; 0; 0; 0];  % [x, y, z, unit quat]
body(1).nu = zeros(6,1);
body(1).mu = 0.1;

body(2).radius = 2;
body(2).mass = 4 / 3 * pi * body(2).radius^3;
body(2).type = 'sphere';
body(2).mom = 2 / 5 * body(2).mass * body(2).radius^2 * eye(3, 3);
body(2).u = [0; 4; 3; 1; 0; 0; 0];  % [x, y, z, unit quat]
body(2).nu = zeros(6,1);
body(2).mu = 0.5;

body(3).radius = 0.5;
body(3).mass = 4 / 3 * pi * body(3).radius^3;
body(3).type = 'sphere';
body(3).mom = 2 / 5 * body(3).mass * body(3).radius^2 * eye(3, 3);
body(3).u = [-2; -2; 3; 1; 0; 0; 0];  % [x, y, z, unit quat]
body(3).nu = zeros(6,1);
body(3).mu = 1;
end

function plotAll(time, trajAll, sim)
%    1:1 ---------------- kinetic energy
%    2:2 ---------------- potential energy
%    3:20 --------------- pext
%   21:38 --------------- Gn * pn
%   39:56 --------------- Gf * pf
%   57:74 --------------- nu
%   75:95 --------------- u
%   96:107 -------------- Psi_n
%  108:119 -------------- s
%  120:131 -------------- pn
%  132:131 + 12*sim.nd -- pf
figure;
plot(time,trajAll(1,:), time,trajAll(2,:), time,trajAll(1,:)+trajAll(2,:));
title('Energy vs Time');
legend('Kinetic', 'Potential', 'Total');
figure;
plot(time,trajAll(3:20,:))
title('External Impulse vs Time');
legend('F1x', 'F1y', 'F1z', 'T1x', 'T1y', 'T1z', ...
       'F2x', 'F2y', 'F2z', 'T2x', 'T2y', 'T2z', ...
       'F3x', 'F3y', 'F3z', 'T3x', 'T3y', 'T3z');
figure;
plot(time,trajAll(21:38,:))
title('Net Normal Impulse vs Time');
legend('F1x', 'F1y', 'F1z', 'T1x', 'T1y', 'T1z', ...
       'F2x', 'F2y', 'F2z', 'T2x', 'T2y', 'T2z', ...
       'F3x', 'F3y', 'F3z', 'T3x', 'T3y', 'T3z');
figure;
plot(time,trajAll(39:56,:))
title('Net Friction Impulse vs Time');
legend('F1x', 'F1y', 'F1z', 'T1x', 'T1y', 'T1z', ...
       'F2x', 'F2y', 'F2z', 'T2x', 'T2y', 'T2z', ...
       'F3x', 'F3y', 'F3z', 'T3x', 'T3y', 'T3z');
figure;
plot(time,trajAll(57:62,:))
title('Body1 Velocity vs Time');
legend('vx', 'vy', 'vz', 'wx', 'wy', 'wz');
figure;
plot(time,trajAll(63:68,:))
title('Body2 Velocity vs Time');
legend('vx', 'vy', 'vz', 'wx', 'wy', 'wz');
figure;
plot(time,trajAll(69:74,:))
title('Body3 Velocity vs Time');
legend('vx', 'vy', 'vz', 'wx', 'wy', 'wz');
figure;
plot(time,trajAll(75:81,:))
title('Body1 Configuration vs Time');
legend('x', 'y', 'z', 'ep0', 'ep1', 'ep2', 'ep3');
figure;
plot(time,trajAll(82:88,:))
title('Body2 Configuration vs Time');
legend('x', 'y', 'z', 'ep0', 'ep1', 'ep2', 'ep3');
figure;
plot(time,trajAll(89:95,:))
title('Body3 Configuration vs Time');
legend('x', 'y', 'z', 'ep0', 'ep1', 'ep2', 'ep3');
figure;
plot(trajAll(75,:),trajAll(76,:), trajAll(82,:),trajAll(83,:), trajAll(89,:),trajAll(90,:))
title('Body Trajectories Projected onto (x,y)-Plane');
legend('Sphere1', 'Sphere2', 'Sphere3');
figure;
plot3(trajAll(75,:),trajAll(76,:),trajAll(77,:), ...
     trajAll(82,:),trajAll(83,:),trajAll(84,:), ...
     trajAll(89,:),trajAll(90,:),trajAll(91,:))
title('Body Trajectories in Space');
legend('Sphere1', 'Sphere2', 'Sphere3');
figure;
plot(time,trajAll(96:107,:))
title('Distance Functions vs Time');
legend('D1', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7', 'D8', 'D9', 'D10', 'D11', 'D12');
figure;
plot(time,trajAll(108:119,:))
title('Sliding Indicator Variables vs Time');
legend('s1', 's2', 's3', 's4', 's5', 's6', 's7', 's8', 's9', 's10', 's11', 's12');
figure;
plot(time,trajAll(120:131,:))
title('Normal Impulses vs Time');
legend('pn1', 'pn2', 'pn3', 'pn4', 'pn5', 'pn6', 'pn7', 'pn8', 'pn9', 'pn10', 'pn11', 'pn12');
figure;
plot(time,trajAll(132:131 + 12*sim.nd,:))
title('Friction Impulses vs Time');
end

function [grav] = makeGrav
grav = [0; 0; -1];
end

function [pext] = makePext(body, sim)
grav = makeGrav;
pext = sim.h * [body(1).mass * grav;  zeros(3,1);
                body(2).mass * grav;  zeros(3,1);
                body(3).mass * grav;  zeros(3,1)];
end