function simSpheresStudentVer2
% Simulate three sphere in a "bowl" formed by three planar half-spaces.
% The number of sphere is fixed at three and the number of half-spaces is
% also fixed at three.  There are 12 pairs of possible contacts.  The gap
% functions can be written in closed form and can be applied at every time
% step, so this is done to simplify the code.

clear all;  % Kill values of persistent variables.

%-----------------------------------------
% Set up params that define the simulation
sim.t0 = 0;      % Initial simluation time
sim.time = 0;    % Current simluation time
sim.h = 0.1;     % Simulation time step
sim.T = 150;     % Final simulation time  
sim.nd = 3;      % Number of friction directions.  CAN BE VARIED!!
sim.fps = 3;     % Number of frames to draw per second of simulation
sim.frmNum = 0;  % Frame counter
sim.solver = 'lemke';
% sim.solver = 'pathlcp';

%-----------------------------------------
% Set up body data structure.  This also sets the initial conditions.
nBodies = 3;    % Fixed number of bodies
nConts = 12;    % Fixed number of contacts (i.e., number of body pairs)
[body] = initBodies();
nu_ell = [body(1).nu; body(2).nu; body(3).nu];
u_ell =  [body(1).u;  body(2).u;  body(3).u];
drawScene(body);   % Draw initial configuration of system

%-----------------------------------------
% Create arrays of solution for ploting trajectories
%    1:1 ---------------- kinetic energy
%    2:2 ---------------- potential energy
%    3:20 --------------- pext
%   21:38 --------------- Gn * pn
%   39:56 --------------- Gf * pf
%   57:74 --------------- nu
%   75:95 --------------- u
%   96:107 -------------- Psi_n
%  108:119 -------------- s
%  120:131 -------------- pn
%  132:131 + 12*sim.nd -- pf
nRows =  nConts * (3 + sim.nd) + 4 * (6 * nBodies) + 7 * nBodies + 2;
time = sim.t0 : sim.h : sim.T;
nCols = length(time);
trajAll = zeros(nRows, nCols);

%-----------------------------------------
% Store the initial condition
% Kinetic energy
M = makeM(body); % Because bodies are spheres, M is constant for all time.
Minv = inv(M);   % Also constant
KE = nu_ell' * M * nu_ell / 2;
trajAll(1,1) = KE;

% Potential energy
grav = makeGrav;
PE = -grav' * (body(1).u(1:3) * body(1).mass ...
             + body(2).u(1:3) * body(2).mass ...
             + body(3).u(1:3) * body(3).mass);
trajAll(2,1) = PE;

% External Impulse
trajAll(3:20, 1) = makePext(body, sim);

% velocities and configurations of bodies
trajAll(57:74, 1) = nu_ell;
trajAll(75:95, 1) = u_ell;

% Distances at contacts
Psi = makePsi(body, sim);
trajAll(96:107, 1) = Psi.N;

%-----------------------------------------
% Start the simulation
% Because all contacts are tracked every time step, U and E are constant
U = makeU(body);  % U is constant for entire simulation
E = makeE(sim);   % E is constant for entire simulation
lcpSolnLast = zeros(12*(2+sim.nd), 1);
for i = 1 : nCols 
    [Gn, Gf] = makeG(body, sim); 
    Psi = makePsi(body, sim);
    pext = makePext(body, sim);       % Input to drive the dynamics
    V = makeV(body);                  % Needed for configuration update

    % Formulate standard LCP here
    MM = [];
    qq = [];
    
    % Guess solution of all zeros
    lcpSoln0 = zeros();
    
    % Call the solver
    switch sim.solver
        case 'lemke'
            [lcpSoln,err] = lemke(MM,qq,lcpSoln0); % Solve the subproblem
        case 'pathlcp' 
            [lcpSoln,mu] = pathlcp(MM,qq,[],[],lcpSoln0);
    end
    % Check error
%     if norm(err) > 1e-10
%         fprintf(strcat(solver, ':  LCP number = %5d    error = %12.6g\n'), i, err);
%     end

    % Extract solution
    pn_ellp1 = lcpSoln();
    pf_ellp1 = lcpSoln();
    s_ellp1  = lcpSoln();
    
    % Update velocities and configurations of the bodies and the time.
    nu_ellp1 = [];
    u_ellp1 = [];

    % Normalize quaternions and update the body data structure
    for j = 1 : 3
        % Renormalize unit quaternions first
        quat = u_ellp1();   % Pull out quaternion from solution
        uQuat = [];         % Normalize it
        u_ellp1() = uQuat;  % Put it back
        % Update body data structure
        body(j).u  =  u_ellp1(j*7-6 : j*7);
        body(j).nu = nu_ellp1(j*6-5 : j*6);
    end
    
    % Update the trajectory information
    % Kinetic energy
    KE = [];

    % Potential energy
    grav = makeGrav;
    PE = [];
    
    % Net contact impulses
    netNormImp = [];
    netFricImp = [];
    
    % Distances at contacts
    Psi = makePsi(body, sim);

    % Update trajectory array with the new values.
    %    1:1 ---------------- kinetic energy
    %    2:2 ---------------- potential energy
    %    3:20 --------------- pext
    %   21:38 --------------- Gn * pn
    %   39:56 --------------- Gf * pf
    %   57:74 --------------- nu
    %   75:95 --------------- u
    %   96:107 -------------- Psi_n
    %  108:119 -------------- s
    %  120:131 -------------- pn
    %  132:131 + 12*sim.nd -- pf
    trajAll(:,i) = [KE;  PE;  pext;  netNormImp;  netFricImp; nu_ellp1; 
                    u_ellp1;  Psi.N;  s_ellp1;  pn_ellp1;  pf_ellp1];

    % Prepare for next iteration of the simulation
    nu_ell = nu_ellp1;
    u_ell  = u_ellp1;
    sim.time = sim.time + sim.h;
    if sim.time > 1 / sim.fps
        drawScene(body);
        sim.frmNum = sim.frmNum + 1;
    end
end
plotAll(time, trajAll, sim);
end